import json
from typing import Optional, Iterator
from typing import Any, List, Dict
from dwai.core.base_completions import BaseCompletions

class Completions(BaseCompletions):

    def run(self, prompt: str, message: List[Dict[str, Any]], stream: bool, **kwargs: Any):

        headers = dict()

        username, bot_name = "guest", "智能机器人"
        message.append({
            "sender_type": "USER",
            "sender_name": username,
            "text": prompt
        })

        data = {
            "bot_setting": [
                {
                    "bot_name": bot_name,
                    "content": "我是一个万能的机器人"
                }
            ],
            "messages": message,
            "reply_constraints": {
                "sender_type": "BOT",
                "sender_name": bot_name
            },
            **kwargs
        }

        resp = self.do_http(
            "/minimax/v1/text/chatcompletion_pro",
            headers,
            data,
            stream)

        if stream:
            return (json.loads(line) for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return json.loads(resp.text)

    def parse_response_stream(self, resp: Iterator[bytes]) -> Iterator[str]:
        for line in resp:
            _line = self.parse_stream_line(line)
            if _line is not None:
                yield _line

    @staticmethod
    def parse_stream_line(line: bytes) -> Optional[str]:
        if line:
            if line.strip() == b"data:[DONE]":
                return None
            if line.startswith(b"data:"):
                line = line[len(b"data:"):]
                return line.decode("utf-8")
            else:
                return None
        return None



import json
from typing import Optional, Iterator, List, Any, Dict

from dwai.core.base_completions import BaseCompletions

class Completions(BaseCompletions):

    def run(self, prompt: str, message: List[Dict[str, Any]], stream: bool, **kwargs: Any):

        headers = dict()
        if stream:
            headers["Accept"] = "text/event-stream"

        self.validate(prompt=prompt)

        message.append({
            "role": "user",
            "content": prompt
        })

        data = {
            "messages": message,
            **kwargs
        }

        resp = self.do_http(
            "/fangzhou/chat/completion",
            headers,
            data,
            stream)

        if stream:
            return (line for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return resp.text

    def validate(self, prompt: str):

        if not self.api_base:
            raise ValueError("you need to get api base")

        if not self.api_key:
            raise ValueError("you need to set DW_API_KEY before calling api")

        if not prompt:
            raise ValueError("prompt is required")

    def parse_response_stream(self, resp: Iterator[bytes]) -> Iterator[str]:
        for line in resp:
            _line = self.parse_stream_line(line)
            if _line is not None:
                yield _line

    @staticmethod
    def parse_stream_line(line: bytes) -> Optional[str]:
        if line:
            if line.strip() == b"data: [DONE]":
                return None
            if line.startswith(b"data: "):
                line = line[len(b"data: "):]
                return line.decode("utf-8")
            else:
                return None
        return None



from dwai.llm.bl import AliBlLLM
from dwai.llm.tx import TiOneLLM
from dwai.llm.pangu import HwPanguLLM
from dwai.llm.minimax import MiniMaxLLM
from dwai.llm.azure import AzureChatOpenAI
from dwai.llm.zhipu import ZhiPuLLM

DW_LLM_MAP = {
    "ali.qwen": AliBlLLM,
    "hw.pangu": HwPanguLLM,
    "ti.one": TiOneLLM,
    "mini.max": MiniMaxLLM,
    "zhipu.ai": ZhiPuLLM,
    "azure": AzureChatOpenAI
}

default_api_key = "dw-BBAa68XBJUqiIj6xUQcC0KREnqmt5mPKQ52wkylD-Tw"
default_api_base = "https://dwai.shizhuang-inc.com"

class DwlLm(object):
    """获取模型参数"""

    def __call__(self, model: str, api_key: str = default_api_key, api_base: str = default_api_base, stream: bool = False, **kwargs):

        if model not in DW_LLM_MAP.keys():
            raise RuntimeError("NOT FOUND DW LLM MODEL")

        llm_model = DW_LLM_MAP.get(model)
        parameters = kwargs if kwargs else {}

        if model == "azure":
            return llm_model(
                deployment_name=kwargs.get("deployment_name"),
                temperature=kwargs.get("temperature"),
                openai_api_version=kwargs.get("openai_api_version"),
                openai_api_key=api_key,
                openai_api_base=api_base,
                openai_api_type=kwargs.get("openai_api_type"),
                openai_organization=kwargs.get("openai_organization"),
                streaming=stream
            )

        return llm_model(
            api_key=api_key,
            api_base=api_base,
            stream=stream,
            parameters=parameters,
        )


def load_dw_llm(model: str, api_key: str = default_api_key, api_base: str = default_api_base, stream: bool = False, **kwargs):

    if model not in DW_LLM_MAP.keys():
        raise RuntimeError("NOT FOUND DW LLM MODEL")

    llm_model = DW_LLM_MAP.get(model)
    parameters = kwargs if kwargs else {}

    if model == "azure":
        return llm_model(
            deployment_name=kwargs.get("deployment_name"),
            temperature=kwargs.get("temperature"),
            openai_api_version=kwargs.get("openai_api_version"),
            openai_api_key=api_key,
            openai_api_base=api_base,
            openai_api_type=kwargs.get("openai_api_type"),
            openai_organization=kwargs.get("openai_organization"),
            streaming=stream
        )

    return llm_model(
        api_key=api_key,
        api_base=api_base,
        stream=stream,
        parameters=parameters,
    )



import json
import dwai
from jsonpath_ng import parse
from dwai.azure.model_api.completions import AzureCompletions
from dwai.bailian.model_api.completions import Completions
from dwai.mini.model_api.completions import MiniCompletions
from dwai.pangu.model_api.completions import PanGuCompletions
from dwai.tione.v20211111.tione_client import TioneClient
from dwai.zhipuai.model_api.api import ModelAPI


def azure_qa(**kwargs):
    chat = AzureCompletions()
    json_output = chat.call(prompt=kwargs.get('prompt'), model=kwargs.get('model'), temperature=kwargs.get('temperature'), stream=False)
    matches = parse('$.choices[0].message.content').find(json_output)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


def mini_qa(**kwargs):
    chat = MiniCompletions()
    json_output = chat.call(prompt=kwargs.get('prompt'), model=kwargs.get('model'), temperature=kwargs.get('temperature'), stream=False)
    matches = parse('$.choices[0].messages[0].text').find(json_output)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


def dwai_bailian_qa(**kwargs):
    chat = Completions()
    json_output = chat.call(app_id="1e4ddc3659324ad0b8a0039230f1dba3", prompt=kwargs.get('prompt'),
                            model=kwargs.get('model'))
    matches = parse('$.output.text').find(json_output)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


def zhipuai_chatglm_std(**kwargs):
    model = ModelAPI()
    json_output = model.invoke(model=kwargs.get('model'), prompt=[{"role": "user", "content": kwargs.get('prompt')}],
                               top_p=kwargs.get('top_p', 0.7), temperature=kwargs.get('temperature', 0.9))
    matches = parse('$.data.choices[0].content').find(json_output)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


def pangu_completions(**kwargs):
    chat = PanGuCompletions()
    json_output = chat.call(max_tokens=kwargs.get('max_tokens', 600), prompt="",
                            messages=[{"role": "user", "content": kwargs.get('prompt')}],
                            temperature=kwargs.get('temperature', 0.9), model=kwargs.get('model'))
    matches = parse('$.choices[0].message.content').find(json_output)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


def tione_chat_completion(**kwargs):
    chat = TioneClient()
    content = chat.ChatCompletion(content=kwargs.get('prompt'), model=kwargs.get('model')).to_json_string()
    json_obj = json.loads(content)
    matches = parse('$.Choices[0].Message.Content').find(json_obj)
    if len(matches) > 0:
        return {"output": {"text": matches[0].value}, "Success": True}
    else:
        return {}


class UnifiedSDK:
    def __init__(self):

        dwai.api_base_china = "https://dwai.shizhuang-inc.com"
        dwai.api_base_singapore = "https://openai.shizhuang-inc.com"

        self.route_map = {
            'alibaba': dwai_bailian_qa,
            'zhipu': zhipuai_chatglm_std,
            'huawei': pangu_completions,
            'minimax': mini_qa,
            'azure': azure_qa,
            'tencent': tione_chat_completion
        }

    def call(self, cloud, **kwargs):
        func = self.route_map.get(cloud)
        if func:
            return func(**kwargs)
        else:
            raise ValueError(f"Unknown cloud: {cloud}")


dwai.api_key = "dw-BBAa68XBJUqiIj6xUQcC0KREnqmt5mPKQ52wkylD-Tw"

if __name__ == '__main__':
    sdk = UnifiedSDK()
    print(">>>")

    data1 = """

        你是一个拥有丰富经验的SRE排障专家，请用中文与我沟通。
        我们现在遇到了一个故障：

        [故障描述]
        sls日志无法采集

        [历史CASE参考]
        问题描述: 业务反馈 algoqc-csprd-shoes-img-verify 应用刚上线，在阿里云后台配置的sls日志采集，从2023年08月14号中午一直没有日志展示
        ------------------------------------
        原因分析:
        原因1,该场景权重占比1%
        云商的logtail扫描没有过滤NAS卷的能力
        原因2,该场景权重占比2%
        业务方错误配置NAS卷挂载点，挂载到了/logs，被多台节点的logtail同时采集，造成IO阻塞
        ------------------------------------
        问题总结:部分业务把NAS卷挂载到了/logs目录，导致多台节点的logtail同时向NAS发起IO请求，出现了IO阻塞。logtail 导致日志采集无法正常工作。
        ------------------------------------
        解决方案:阿里云改造logtail能力，使之可以支持绕开NAS卷扫描，化解NAS扫描的IO压力

        [我的要求]
        请仔细参考这个case，请结合你多年的经验。帮我推断当前故障，值得排查的方向。
        尽可能以 Markdown 格式输出，排版要精美，避免不必要的换行。

      """
    data = """

    你是一个拥有丰富经验的SRE排障专家，请用中文与我沟通。
    我们现在遇到了一个故障：

    [故障描述]
    sls日志无法采集


    [历史CASE]：仅供参考

问题描述: 业务反馈 algoqc-csprd-shoes-img-verify 应用刚上线，在阿里云后台配置的sls日志采集，从2023年08月14号中午一直没有日志展示
------------------------------------
原因分析:
原因1,该场景权重占比1%
云商的logtail扫描没有过滤NAS卷的能力
原因2,该场景权重占比2%
业务方错误配置NAS卷挂载点，挂载到了/logs，被多台节点的logtail同时采集，造成IO阻塞
------------------------------------
问题总结:部分业务把NAS卷挂载到了/logs目录，导致多台节点的logtail同时向NAS发起IO请求，出现了IO阻塞。logtail 导致日志采集无法正常工作。
------------------------------------
解决方案:阿里云改造logtail能力，使之可以支持绕开NAS卷扫描，化解NAS扫描的IO压力


    [我的要求]
    请结合从前case和你多年的经验，进行深度思考。给出概率较高的详细 排障方向 和 解决思路。并且分别给出概率值！
    不用输出我的上下文，尽可能以 Markdown 格式输出，排版简洁美观

    """

    #
    # # 测试微软
    # resp = sdk.call('azure', model="gpt-35-turbo-16k", prompt=data, temperature=0.1)
    # re = resp.get("output").get("text")
    # print(re.replace("\\n", "\n"))

    # # 测试mini
    # resp = sdk.call('minimax', model="abab5.5-chat", prompt="你的参数量大概是多少？", temperature=0.1)
    # print(resp)
    #
    # # 测试dwai
    resp = sdk.call('alibaba', model="qwen-plus-v1", prompt="你的参数量大概是多少？")
    print(resp)

    # 测试zhipuai
    # resp = sdk.call('zhipu', model="chatglm_std", prompt=data, top_p=0.7, temperature=0.9)
    # re = resp.get("output").get("text")
    # print(re.replace("\\n", "\n"))

    #
    # # 测试pangu
    # resp = sdk.call('huawei', model="default", prompt="你的参数量大概是多少？", max_tokens=600, temperature=0.9)
    # print(resp)
    #
    # # 测试tione
    # resp = sdk.call('tencent', model="default", prompt="你的参数量大概是多少？")
    # print(resp)



import json
from typing import Optional, Iterator, List, Any

from dwai.core.base_completions import BaseCompletions
from dwai.bailian.model_api.models import ChatQaMessage

class Completions(BaseCompletions):

    def run(self, prompt: str, history: List[Any], stream: bool, **kwargs: Any):

        headers = dict()
        if stream:
            headers["Accept"] = "text/event-stream"

        self.validate(prompt=prompt)

        h = []
        if history is not None:
            for v in history:
                h.append(v.to_dict())

        data = {
            "Stream": stream,
            "Prompt": prompt,
            **kwargs
        }

        resp = self.do_http(
            "/bailian/v2/app/completions",
            headers,
            data,
            stream)

        if stream:
            return (json.loads(line) for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return json.loads(resp.text)

    def run_v1(self, prompt: str, history: List[ChatQaMessage], stream: bool, **kwargs: Any):

        headers = dict()

        if stream:
            headers["Accept"] = "text/event-stream"

        self.validate(prompt=prompt)

        h = []
        if history is not None:
            for v in history:
                h.append(v.to_dict())

        data = {
            "model": "qwen-plus-v1",
            "input": {
                "prompt": prompt,
                "history": h
            },
            "parameters": kwargs
        }

        resp = self.do_http(
            "/bailian/services/aigc/text-generation/generation",
            headers,
            data,
            stream)

        if stream:
            return (json.loads(line) for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return json.loads(resp.text)

    def validate(self, prompt: str):

        if not self.api_base:
            raise ValueError("you need to get api base")

        if not self.api_key:
            raise ValueError("you need to set DW_API_KEY before calling api")

        if not prompt:
            raise ValueError("prompt is required")

    def parse_response_stream(self, resp: Iterator[bytes]) -> Iterator[str]:
        for line in resp:
            _line = self.parse_stream_line(line)
            if _line is not None:
                yield _line

    @staticmethod
    def parse_stream_line(line: bytes) -> Optional[str]:
        if line:
            if line.strip() == b"data: [DONE]":
                return None
            if line.startswith(b"data: "):
                line = line[len(b"data: "):]
                return line.decode("utf-8")
            else:
                return None
        return None



import json
from typing import Optional, Iterator
from typing import Any, List, Dict
from dwai.core.base_completions import BaseCompletions

class Completions(BaseCompletions):

    def run(self, prompt: str, message: List[Dict[str, Any]], stream: bool, **kwargs: Any):

        headers = dict()

        message.append({"role": "user", "content": prompt})
        model = "chatglm_std"
        if "model" in kwargs:
            model = kwargs.get("model")

        action = "invoke"
        if "action" in kwargs:
            action = kwargs.get("action")
            del kwargs["action"]

        data = {
            "prompt": message,
            **kwargs
        }

        url = "/zhipu/api/paas/v3/model-api/%s/%s" % (model, action)

        resp = self.do_http(url, headers, data, stream)

        if stream:
            return (json.loads(line) for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return json.loads(resp.text)

    def parse_response_stream(self, resp: Iterator[bytes]) -> Iterator[str]:
        for line in resp:
            _line = self.parse_stream_line(line)
            if _line is not None:
                yield _line

    @staticmethod
    def parse_stream_line(line: bytes) -> Optional[str]:
        if line:
            if line.strip() == b"data:[DONE]":
                return None
            if line.startswith(b"data:"):
                line = line[len(b"data:"):]
                return line.decode("utf-8")
            else:
                return None
        return None



# -*- coding:utf-8 -*-
import json
import logging

import requests
from typing import Dict, Optional, Iterator

import dwai


def post(api_url, params):
    try:
        headers = dict()
        headers["Content-Type"] = "application/json;charset=UTF-8"
        headers["Authorization"] = dwai.api_key

        resp = requests.post(
            url=api_url, data=json.dumps(params), headers=headers, timeout=dwai.api_timeout_seconds
        )
        if requests.codes.ok != resp.status_code:
            raise Exception("响应异常：" + resp.content)
        return json.loads(resp.text)
    except Exception as e:
        logging.exception("请求异常", e)


def stream(api_url, params):
    try:
        headers = dict()
        headers["Authorization"] = dwai.api_key

        resp = requests.post(
            api_url,
            stream=True,
            headers=headers,
            json=params,
            timeout=dwai.api_timeout_seconds,
        )
        if requests.codes.ok != resp.status_code:
            raise Exception("请求异常")
        return resp
    except Exception as e:
        logging.exception("请求异常", e)


def get(api_url):
    try:
        headers = dict()
        headers["Authorization"] = dwai.api_key
        resp = requests.get(api_url, headers=headers, timeout=dwai.api_timeout_seconds)
        if requests.codes.ok != resp.status_code:
            raise Exception("响应异常：" + resp.content)
        return json.loads(resp.text)
    except Exception as e:
        logging.exception("请求异常", e)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

import pytest
import openai
from devtools_testutils import AzureRecordedTestCase
from conftest import configure, AZURE, OPENAI, ALL


class TestDallE(AzureRecordedTestCase):

    @pytest.mark.parametrize("api_type", ALL)
    @configure
    def test_image_create(self, azure_openai_creds, api_type):
        image = openai.Image.create(
            prompt="a cute baby seal"
        )
        assert image.created
        assert len(image.data) == 1
        assert image.data[0].url

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_image_create_n(self, azure_openai_creds, api_type):
        image = openai.Image.create(
            prompt="a cute baby seal",
            n=2
        )
        assert image.created
        assert len(image.data) == 2
        for img in image.data:
            assert img.url

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_image_create_size(self, azure_openai_creds, api_type):
        image = openai.Image.create(
            prompt="a cute baby seal",
            size="256x256"
        )
        assert image.created
        assert len(image.data) == 1
        assert image.data[0].url

    @pytest.mark.parametrize("api_type", [OPENAI])
    @configure
    def test_image_create_response_format(self, azure_openai_creds, api_type):
        image = openai.Image.create(
            prompt="a cute baby seal",
            response_format="b64_json"  # No Azure support yet
        )
        assert image.created
        assert len(image.data) == 1
        assert image.data[0].b64_json

    @pytest.mark.parametrize("api_type", [AZURE, OPENAI])
    @configure
    def test_image_create_user(self, azure_openai_creds, api_type):
        image = openai.Image.create(
            prompt="a cute baby seal",
            user="krista"
        )
        assert image.created
        assert len(image.data) == 1
        assert image.data[0].url



# -*- coding: utf8 -*-

import json
from typing import Iterator, Optional

from dwai.tione.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
from dwai.tione.common.base_completions import BaseCompletions
from dwai.tione.v20211111 import models

class TioneClient(BaseCompletions):
    _apiVersion = '2021-11-11'
    _endpoint = 'tione.tencentcloudapi.com'
    _service = 'tione'

    def call(self,
             action: str = None,
             max_tokens: str = None,
             content: str = None,
             top_p: float = 0.0,
             stream: bool = False):
        resp = self.reqeust(
            action=action,
            max_tokens=max_tokens,
            content=content,
            top_p=top_p,
            stream=stream,
        )

        if stream:
            return (json.loads(line) for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return json.loads(resp.text)

    def ChatCompletion(self, model: str, content, stream=False, top_p=None, max_tokens=None):
        try:
            response = self.call(
                "ChatCompletion",
                content=content,
                stream=stream,
                top_p=top_p,
                max_tokens=max_tokens
            )
            model = models.ChatCompletionResponse()
            model._deserialize(response)
            return model
        except Exception as e:
            if isinstance(e, TencentCloudSDKException):
                raise
            else:
                raise TencentCloudSDKException(type(e).__name__, str(e))

    def parse_response_stream(self, resp: Iterator[bytes]) -> Iterator[str]:
        for line in resp:
            _line = self.parse_stream_line(line)
            if _line is not None:
                yield _line

    @staticmethod
    def parse_stream_line(line: bytes) -> Optional[str]:
        if line:
            if line.strip() == b"data:[DONE]":
                return None
            if line.startswith(b"data:"):
                line = line[len(b"data:"):]
                return line.decode("utf-8")
            else:
                return None
        return None



import json
from typing import Dict, Optional, Iterator, List

from dwai.mini.model_api.base_completions import BaseCompletions


class MiniCompletions(BaseCompletions):
    def call(self,
             prompt: str,
             model: str,
             temperature: float,
             stream: bool = False):
        resp = self.reqeust(prompt=prompt,
                            stream=stream,
                            model=model,
                            temperature=temperature)

        if stream:
            return (json.loads(line) for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return json.loads(resp.text)

    def parse_response_stream(self, resp: Iterator[bytes]) -> Iterator[str]:
        for line in resp:
            _line = self.parse_stream_line(line)
            if _line is not None:
                yield _line

    @staticmethod
    def parse_stream_line(line: bytes) -> Optional[str]:
        if line:
            if line.strip() == b"data:[DONE]":
                return None
            if line.startswith(b"data:"):
                line = line[len(b"data:"):]
                return line.decode("utf-8")
            else:
                return None
        return None
